package CH.ifa.draw.standard;

import CH.ifa.draw.DrawPlugin;

import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.application.MenuManager;

import CH.ifa.draw.figures.TextFigure;

import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;

import CH.ifa.draw.util.GUIProperties;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Enumeration;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;


/** Constructs a SearchFrame or SearchAndReplaceFrame
    according to the selected mode and displays it.
*/

/*
 * this should be changed so that it doesnt need the application
 * as an argument to the constructor, instead retrieving it
 * via the plugin.
 * As well, it should register as a closelistener...
 *
 */
public class SearchReplaceFrame {
    public static final int SEARCHMODE = 0;
    public static final int SEARCHREPLACEMODE = 1;

    // private DrawApplication application;
    private Enumeration drawings;
    private FigureEnumeration figures;
    private Drawing drawing;
    private JFrame frame;
    private boolean stringMatcherValid = false;
    private StringMatcher substringMatcher;
    private int mode;

    public SearchReplaceFrame(int aMode) {
        String frameTitle;
        String buttonTitle;

        // this.application = application;
        mode = aMode;

        if (mode == SEARCHMODE) {
            frameTitle = "Search";
        } else {
            frameTitle = "Search & Replace";
        }


        frame = new JFrame(frameTitle);

        if (!GUIProperties.avoidFrameReshape()) {
            frame.setSize(600, 200);
        }
        GridBagLayout gridBag = new GridBagLayout();
        frame.getContentPane().setLayout(gridBag);

        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;

        JLabel label = new JLabel("Search for:");
        gridBag.setConstraints(label, c);
        frame.getContentPane().add(label);

        c.gridwidth = GridBagConstraints.REMAINDER; //end row
        final JTextField searchTextField = new JTextField("", 30);
        gridBag.setConstraints(searchTextField, c);
        searchTextField.addKeyListener(new KeyAdapter() {
                public void keyTyped(KeyEvent e) {
                    reset(); //search string is modified: reset search process
                }
            });
        frame.getContentPane().add(searchTextField);

        final JTextField replaceTextField = new JTextField("", 30);
        if (mode == SEARCHREPLACEMODE) {
            c.gridwidth = 1;
            label = new JLabel("Replace with:");
            gridBag.setConstraints(label, c);
            frame.getContentPane().add(label);

            c.gridwidth = GridBagConstraints.REMAINDER; //end row
            gridBag.setConstraints(replaceTextField, c);
            replaceTextField.addKeyListener(new KeyAdapter() {
                    public void keyTyped(KeyEvent e) {
                        stringMatcherValid = false; //replace string is modified
                    }
                });
            frame.getContentPane().add(replaceTextField);
        }

        c.weightx = 1;


        if (mode == SEARCHMODE) {
            buttonTitle = "Search";
        } else {
            buttonTitle = "Search & Replace";
        }
        JButton searchButton = new JButton(buttonTitle);
        gridBag.setConstraints(searchButton, c);
        frame.getContentPane().add(searchButton);
        searchButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (!stringMatcherValid) {
                        if (mode == SEARCHMODE) {
                            substringMatcher = new SubstringMatcher(searchTextField
                                                                    .getText(),
                                                                    null);
                        } else {
                            substringMatcher = new SubstringMatcher(searchTextField
                                                                    .getText(),
                                                                    replaceTextField
                                                                    .getText());
                        }
                        stringMatcherValid = true;
                    }
                    searchInAllDrawings();
                }
            });

        c.gridwidth = GridBagConstraints.REMAINDER; //end row
        JButton cancelButton = new JButton("Cancel");
        gridBag.setConstraints(cancelButton, c);
        frame.getContentPane().add(cancelButton);
        cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    cancel();
                }
            });


        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent w) {
                    frame.setVisible(false);
                    MenuManager.getInstance().getWindowsMenu().removeFrame(frame);
                }
            });


        //frame.validate();
        frame.pack();

        reset();

    }

    /**
       Handles a found TextFigure wrt. the actual mode. In case of a
       SEARCHREPLACEMODE this is done by prompting the user with the
       replacement.
    */
    private void handleFoundFigure(TextFigure foundFigure) {
        if (mode == SEARCHREPLACEMODE) {
            String foundText = foundFigure.getText();
            String replacement = substringMatcher.replacement(foundText);
            int answer = JOptionPane.showConfirmDialog(frame,
                                                       "Replace:\n" + foundText
                                                       + "\nwith:\n"
                                                       + replacement,
                                                       "Confirm Replacement",
                                                       JOptionPane.YES_NO_CANCEL_OPTION);

            switch (answer) {
            case JOptionPane.CANCEL_OPTION:
                reset();
                setVisible(false);
                return;
            case JOptionPane.YES_OPTION:
                DrawApplication application = DrawPlugin.getGui();
                application.prepareUndoSnapshot();
                foundFigure.setText(replacement);
                application.commitUndoSnapshot();
                application.view().checkDamage();
                break;
            case JOptionPane.NO_OPTION:
                break;
            }
        }
    }

    /**
     * Searches for a matching string in @see CH.ifa.draw.figures.TextFigure s
     *   and calls @see handleFoundFigure(TextFigure t) for each match.
     *
     * @author  Michael Koehler, Heiko Roelke
     *
     *
     */
    private void searchInAllDrawings() {
        TextFigure foundFigure = null;
        do {
            if (drawing == null) { //start a new search in first/next drawing
                if (drawings.hasMoreElements()) {
                    drawing = (Drawing) drawings.nextElement();
                    figures = drawing.figures();
                } else {
                    Toolkit.getDefaultToolkit().beep();
                    return; //no more drawings :((   
                }
            }

            foundFigure = searchInOneDrawing();
            if (foundFigure != null) {
                DrawApplication application = DrawPlugin.getGui();
                application.selectOffendingElements(new FigureException("Heureka",
                                                                        "Ich hab's gefunden",
                                                                        drawing,
                                                                        foundFigure));
                if (mode == SEARCHMODE) {
                    return;
                } else {
                    handleFoundFigure(foundFigure);
                }
            } else {
                // no more figures in this drawing, next one.
                drawing = null;
            }
        } while (drawings.hasMoreElements());
        if (!figures.hasMoreElements() & foundFigure == null) {
            Toolkit.getDefaultToolkit().beep();
            // optional: reset();
        }
    }

    private TextFigure searchInOneDrawing() {
        while (figures.hasMoreElements()) {
            Figure figure = figures.nextFigure();
            if (figure instanceof TextFigure) {
                TextFigure textFigure = (TextFigure) figure;
                String figureText = textFigure.getText();
                if (substringMatcher.matches(figureText)) {
                    return textFigure; //Heureka!
                }
            }
        }
        return null;
    }

    public void reset() {
        DrawApplication application = DrawPlugin.getGui();
        if (application != null) {
            drawings = application.drawings();
            drawing = null;
            figures = null;
            stringMatcherValid = false;
        }
    }

    public void cancel() {
        this.setVisible(false);
    }

    public void setVisible(boolean b) {
        frame.setVisible(b);
        if (b) {
            MenuManager.getInstance().getWindowsMenu().addFrame(DrawPlugin.WINDOWS_CATEGORY_TOOLS,
                                                                frame);
        } else {
            MenuManager.getInstance().getWindowsMenu().removeFrame(frame);
        }
    }
}