package de.renew.gui;

import de.renew.engine.searcher.AbstractOccurrence;
import de.renew.engine.searcher.OccurrenceDescription;
import de.renew.engine.searcher.Searcher;
import de.renew.engine.searcher.VariableMapperCopier;

import de.renew.net.TransitionInstance;

import de.renew.unify.Impossible;

import java.util.Collection;
import java.util.Collections;


/**
 * Occurrence of a breakpoint inscription.
 *
 * BreakpointOccurrence.java
 * Created: Wed May 31  2000
 * @author Michael Duvigneau
 *
 * @see BreakpointInscription
 **/
class BreakpointOccurrence extends AbstractOccurrence {
    private GlobalTransitionBreakpoint breakpoint;

    /**
     * Occurres when a defined breakpoint is reached.
     *
     * @param breakpoint
     *   the defined breakpoint.
     * @param tInstance
     *   The transition instance this inscription occurs at.
     *   The breakpoint has to know at which transition instance
     *   it occurrs to be able to inform the user.
     **/
    public BreakpointOccurrence(GlobalTransitionBreakpoint breakpoint,
                                TransitionInstance tInstance) {
        super(tInstance);
        this.breakpoint = breakpoint;
    }

    /**
     * @return    empty enumeration
     *            (breakpoint holds in all bindings)
     * @exception de.renew.unify.Impossible
     *            never?
     */
    public Collection makeBinders(Searcher searcher) throws Impossible {
        return Collections.EMPTY_SET;
    }

    /**
     * @return one executable of this breakpoint
     * @see BreakpointExecutable
     **/
    public Collection makeExecutables(VariableMapperCopier copier) {
        return Collections.singleton(new BreakpointExecutable(breakpoint,
                                                              getTransition()));
    }

    public OccurrenceDescription makeOccurrenceDescription(VariableMapperCopier copier) {
        return null;
    }
}