package de.renew.gui.maria;

import de.renew.shadow.ShadowArc;
import de.renew.shadow.ShadowDeclarationNode;
import de.renew.shadow.ShadowInscription;
import de.renew.shadow.ShadowNet;
import de.renew.shadow.ShadowPlace;
import de.renew.shadow.ShadowTransition;

import java.io.PrintWriter;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;


public class MariaExporter {
    private PrintWriter writer;

    public MariaExporter(PrintWriter writer) {
        this.writer = writer;
    }

    public void export(ShadowNet net) {
        Vector earlyNetInscr = new Vector();
        Vector lateNetInscr = new Vector();
        Vector places = new Vector();
        Vector transitions = new Vector();
        Hashtable transitionToArcs = new Hashtable();
        Hashtable nodesToEarlyInscrVectors = new Hashtable();
        Hashtable nodesToLateInscrVectors = new Hashtable();

        // Sort shadow net elements.
        Iterator iterator = net.elements().iterator();
        while (iterator.hasNext()) {
            Object elem = iterator.next();
            if (elem instanceof ShadowPlace) {
                places.addElement(elem);
            } else if (elem instanceof ShadowTransition) {
                transitions.addElement(elem);
            } else if (elem instanceof ShadowArc) {
                ShadowArc arc = (ShadowArc) elem;

                // Sort the arc according to the neighboring transition.
                Vector arcVector = (Vector) transitionToArcs.get(arc.transition);
                if (arcVector == null) {
                    arcVector = new Vector();
                    transitionToArcs.put(arc.transition, arcVector);
                }
                arcVector.addElement(arc);
            } else if (elem instanceof ShadowDeclarationNode) {
                ShadowDeclarationNode decl = (ShadowDeclarationNode) elem;
                Vector vec;
                if (isLateNetInscr(decl.inscr)) {
                    vec = lateNetInscr;
                } else {
                    vec = earlyNetInscr;
                }
                vec.addElement(decl.inscr);
            }
        }


        // Output Maria code.
        // Early net inscriptions (declarations) first.
        for (int i = 0; i < earlyNetInscr.size(); i++) {
            handleEarlyNetInscr((String) earlyNetInscr.elementAt(i));
        }


        // Places and their inscriptions go next. Places must later
        // be referenced by transitions via arcs.
        for (int i = 0; i < places.size(); i++) {
            ShadowPlace place = (ShadowPlace) places.elementAt(i);
            String name = place.getName();

            Vector capacities = new Vector();
            Vector types = new Vector();
            Vector tokens = new Vector();
            Iterator placeInscriptions = place.elements().iterator();
            while (placeInscriptions.hasNext()) {
                Object elem = placeInscriptions.next();
                if (elem instanceof ShadowInscription) {
                    ShadowInscription inscr = (ShadowInscription) elem;
                    if (inscr.isSpecial()) {
                        if (isCapacity(inscr.inscr)) {
                            capacities.addElement(inscr.inscr);
                        } else {
                            types.addElement(inscr.inscr);
                        }
                    } else {
                        tokens.addElement(inscr.inscr);
                    }
                }
            }
            String[] capacityArr = new String[capacities.size()];
            capacities.copyInto(capacityArr);
            String[] typeArr = new String[types.size()];
            types.copyInto(typeArr);
            String[] tokenArr = new String[tokens.size()];
            tokens.copyInto(tokenArr);
            handlePlace(name, capacityArr, typeArr, tokenArr);
        }


        // Output transitions and all arcs incident to each of the
        // transitions.
        for (int i = 0; i < transitions.size(); i++) {
            ShadowTransition transition = (ShadowTransition) transitions
                                          .elementAt(i);

            Vector early = new Vector();
            Vector late = new Vector();
            Iterator transitionInscriptions = transition.elements().iterator();
            while (transitionInscriptions.hasNext()) {
                Object elem = transitionInscriptions.next();
                if (elem instanceof ShadowInscription) {
                    ShadowInscription inscr = (ShadowInscription) elem;
                    if (isLateTransitionInscr(inscr.inscr)) {
                        late.addElement(inscr.inscr);
                    } else {
                        early.addElement(inscr.inscr);
                    }
                }
            }
            String[] earlyArr = new String[early.size()];
            early.copyInto(earlyArr);
            String[] lateArr = new String[late.size()];
            late.copyInto(lateArr);


            // Collect all inscriptions of all incident arcs and group them
            // according to the places.
            Hashtable inArcs = new Hashtable();
            Hashtable outArcs = new Hashtable();
            Vector arcs = (Vector) transitionToArcs.get(transition);
            for (int j = 0; j < arcs.size(); j++) {
                ShadowArc arc = (ShadowArc) arcs.elementAt(j);
                Iterator arcInscrs = arc.elements().iterator();
                while (arcInscrs.hasNext()) {
                    Object elem = arcInscrs.next();
                    if (elem instanceof ShadowInscription) {
                        ShadowInscription inscr = (ShadowInscription) elem;
                        if (arc.shadowArcType != ShadowArc.ordinary
                                    || arc.placeToTransition) {
                            Vector allInscrs = (Vector) inArcs.get(arc.place);
                            if (allInscrs == null) {
                                allInscrs = new Vector();
                                inArcs.put(arc.place, allInscrs);
                            }
                            allInscrs.addElement(inscr.inscr);
                        }
                        if (arc.shadowArcType != ShadowArc.ordinary
                                    || !arc.placeToTransition) {
                            Vector allInscrs = (Vector) outArcs.get(arc.place);
                            if (allInscrs == null) {
                                allInscrs = new Vector();
                                outArcs.put(arc.place, allInscrs);
                            }
                            allInscrs.addElement(inscr.inscr);
                        }
                    }
                }
            }


            // Output those transition inscriptions that
            // were detected as declarations etc.
            handleStartTransition(transition.getName(), earlyArr);


            // Output all input arcs with all inscriptions.
            // All arc incident to one place and all inscriptions
            // of these arcs are handled jointly.
            Enumeration inPlaces = inArcs.keys();
            while (inPlaces.hasMoreElements()) {
                ShadowPlace place = (ShadowPlace) inPlaces.nextElement();
                Vector inscrs = (Vector) inArcs.get(place);
                String[] inscrArr = new String[inscrs.size()];
                inscrs.copyInto(inscrArr);
                handleInputArc(place.getName(), inscrArr);
            }

            // Output arcs next. Same as before.
            Enumeration outPlaces = outArcs.keys();
            while (outPlaces.hasMoreElements()) {
                ShadowPlace place = (ShadowPlace) outPlaces.nextElement();
                Vector inscrs = (Vector) outArcs.get(place);
                String[] inscrArr = new String[inscrs.size()];
                inscrs.copyInto(inscrArr);
                handleOutputArc(place.getName(), inscrArr);
            }


            // Gate expressions come in last position.
            handleEndTransition(lateArr);
        }

        // Formulas for verification etc.
        for (int i = 0; i < lateNetInscr.size(); i++) {
            handleLateNetInscr((String) lateNetInscr.elementAt(i));
        }
    }

    public boolean isLateNetInscr(String inscr) {
        try {
            return SingleMariaNetCompiler.createParser(inscr).netinscr();
        } catch (ParseException e) {
            return false;
        }
    }

    public boolean isCapacity(String inscr) {
        try {
            return SingleMariaNetCompiler.createParser(inscr).typeorcapacity();
        } catch (ParseException e) {
            return false;
        }
    }

    // This method is only called for inscriptions of transitions.
    public boolean isLateTransitionInscr(String inscr) {
        try {
            return SingleMariaNetCompiler.createParser(inscr).transinscr();
        } catch (ParseException e) {
            return false;
        }
    }

    public void handleEarlyNetInscr(String inscr) {
        writer.println(inscr);
    }

    public String makeTokenList(String[] tokens) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < tokens.length; i++) {
            if (i > 0) {
                buf.append(",\n  ");
            }
            buf.append(' ');
            buf.append(tokens[i]);
        }
        return buf.toString();
    }

    // If many inscriptions are identified as a type,
    // all of them are provided in the types array.
    public void handlePlace(String name, String[] capacities, String[] types,
                            String[] tokens) {
        StringBuffer buf = new StringBuffer("place " + name);
        if (capacities.length != 0) {
            buf.append(' ');
            buf.append(capacities[0]);
        }
        if (types.length != 0) {
            buf.append(' ');
            buf.append(types[0]);
        }
        if (tokens.length != 0) {
            buf.append(':');
            buf.append(makeTokenList(tokens));
        }
        buf.append(';');
        writer.println(buf);
    }

    public void handleStartTransition(String name, String[] inscrs) {
        writer.print("trans " + name);
        for (int i = 0; i < inscrs.length; i++) {
            writer.print('\n' + inscrs[i]);
        }
    }

    public void handleInputArc(String placeName, String[] inscrs) {
        writer.print("\nin { place " + placeName + ":" + makeTokenList(inscrs)
                     + " }");
    }

    public void handleOutputArc(String placeName, String[] inscrs) {
        writer.print("\nout { place " + placeName + ":" + makeTokenList(inscrs)
                     + " }");
    }

    public void handleEndTransition(String[] inscrs) {
        for (int i = 0; i < inscrs.length; i++) {
            writer.print("\n" + inscrs[i]);
        }
        writer.println(";");
    }

    public void handleLateNetInscr(String inscr) {
        writer.println(inscr);
    }
}