/*
 * Created on 17.08.2004
 */
package de.renew.engine.common;

import de.renew.application.SimulatorPlugin;

import de.renew.engine.events.SimulationEvent;

import de.renew.net.NetInstance;
import de.renew.net.PlaceInstance;
import de.renew.net.TransitionInstance;


/**
 * @author Sven Offermann
 */
public class SimulatorEvent {
    private StepIdentifier stepIdentifier = null;
    private SimulationEvent logObject;
    private NetInstance netInstance = null;

    // can be TransitionInstance, PlaceInstance or null
    private Object netElementInstance = null;

    public SimulatorEvent(SimulationEvent traceObject)
            throws Exception {
        this.logObject = traceObject;
        if (SimulatorPlugin.getCurrent() == null) {
            throw new Exception("No simulator running. Cant create Simulator Event.");
        }
        this.stepIdentifier = SimulatorPlugin.getCurrent()
                                             .getCurrentEnvironment()
                                             .getSimulator()
                                             .currentStepIdentifier();
    }

    public SimulatorEvent(StepIdentifier stepIdentifier,
                          SimulationEvent traceObject) {
        this.stepIdentifier = stepIdentifier;
        this.logObject = traceObject;
    }

    public SimulatorEvent(StepIdentifier stepIdentifier,
                          SimulationEvent traceObject,
                          TransitionInstance transitionInstance) {
        this.stepIdentifier = stepIdentifier;
        this.logObject = traceObject;
        this.netElementInstance = transitionInstance;
        if (transitionInstance != null) {
            this.netInstance = transitionInstance.getNetInstance();
        }
    }

    public SimulatorEvent(StepIdentifier stepIdentifier,
                          SimulationEvent traceObject,
                          PlaceInstance placeInstance) {
        this.stepIdentifier = stepIdentifier;
        this.logObject = traceObject;
        this.netElementInstance = placeInstance;
        if (placeInstance != null) {
            this.netInstance = placeInstance.getNetInstance();
        }
    }

    public SimulatorEvent(StepIdentifier stepIdentifier,
                          SimulationEvent traceObject, NetInstance netInstance) {
        this.stepIdentifier = stepIdentifier;
        this.logObject = traceObject;
        this.netInstance = netInstance;
    }

    public StepIdentifier getStep() {
        return this.stepIdentifier;
    }

    public SimulationEvent getMessage() {
        return this.logObject;
    }

    public String toString() {
        String message = "";
        if (this.stepIdentifier.getComponents().length > 0) {
            message = this.stepIdentifier.toString();
        }

        return message + " " + this.logObject.toString();
    }

    public Object getNetElementInstance() {
        return this.netElementInstance;
    }

    public NetInstance getNetInstance() {
        return this.netInstance;
    }
}