package de.renew.engine.searcher;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;


public class TriggerCollection implements Serializable {
    // Keep track of triggering objects.
    private Set triggers;
    private Triggerable triggerable;

    public TriggerCollection(Triggerable triggerable) {
        triggers = new HashSet();
        this.triggerable = triggerable;
    }

    // The following two method must only be accessed through
    // the include and exclude methods of triggerable collections,
    // otherwise deadlocks might occur. Those method will make sure
    // to lock this object.
    public void include(TriggerableCollection trigger) {
        triggers.add(trigger);
    }

    public void exclude(TriggerableCollection trigger) {
        triggers.remove(trigger);
    }

    public void clear() {
        List list = new ArrayList();
        synchronized (this) {
            // Get the triggers.
            list.addAll(triggers);
        }


        // Send the notifications. No need to synchronize here.
        // Better allow concurrent accesses to reduce deadlock potential.
        // The triggerable collections are required to do the locking.
        Iterator iterator = list.iterator();
        while (iterator.hasNext()) {
            TriggerableCollection triggerables = (TriggerableCollection) iterator
                                                 .next();
            triggerables.exclude(triggerable);
        }
    }
}