package de.renew.remote;

import de.renew.application.SimulationEnvironment;

import de.renew.net.Net;
import de.renew.net.NetElementID;
import de.renew.net.Place;
import de.renew.net.Transition;

import java.rmi.RemoteException;

import java.util.Iterator;


/**
 * This class implements the <code>NetAccessor</code> interface
 * and nothing more.
 * <p>
 * </p>
 * NetAccessorImpl.java
 * Created: Thu Jul 13  2000
 * @author Michael Duvigneau
 */
public class NetAccessorImpl extends ObjectAccessorImpl implements NetAccessor {

    /**
     * Creates a new net accessor for the given net.
     *
     * @param net the net to access
     * @exception java.rmi.RemoteException if an RMI failure occured.
     */
    public NetAccessorImpl(Net net, SimulationEnvironment env)
            throws RemoteException {
        super(net, env);
    }

    /* This method is specified by the NetAccessor interface. */
    public String getName() throws RemoteException {
        return ((Net) object).getName();
    }

    // -------------------------------------------- Place handling ----

    /* This method is specified by the NetAccessor interface. */
    public NetElementID[] getPlaceIDs() throws RemoteException {
        Net net = (Net) object;
        synchronized (net) {
            NetElementID[] placeIDs = new NetElementID[((Net) object)
                                                       .placeCount()];
            Iterator places = net.places().iterator();
            for (int i = 0; i < placeIDs.length; i++) {
                placeIDs[i] = ((Place) places.next()).getID();
            }
            return placeIDs;
        }
    }

    /**
     * Return a place accessor for the place with the given
     * ID. Return <code>null</code>, if no such place exists.
     *
     * This method is specified by the NetAccessor interface.
     *
     * <p>
     * An additional remark to the validity of the given id:
     * It is not checked if the set of places of the accessed
     * net has changed since the last call to <code>getPlaceIDs()
     * </code>.
     * </p>
     *
     * @see NetAccessor#getPlace
     */
    public PlaceAccessor getPlace(NetElementID id) throws RemoteException {
        Place result = ((Net) object).getPlaceWithID(id);

        if (result == null) {
            return null;
        }
        return new PlaceAccessorImpl(result, getEnvironment());
    }

    // --------------------------------------- Transition handling ----

    /* This method is specified by the NetAccessor interface. */
    public NetElementID[] getTransitionIDs() throws RemoteException {
        Net net = (Net) object;
        synchronized (net) {
            NetElementID[] transitionIDs = new NetElementID[((Net) object)
                                                            .transitionCount()];
            Iterator transitions = net.transitions().iterator();
            for (int i = 0; i < transitionIDs.length; i++) {
                transitionIDs[i] = ((Transition) transitions.next()).getID();
            }
            return transitionIDs;
        }
    }

    /**
     * Return a transition accessor for the transition with the given
     * ID. Return <code>null</code>, if no such transition exists.
     *
     * This method is specified by the NetAccessor interface.
     *
     * <p>
     * An additional remark to the validity of the given id:
     * It is not checked if the set of transitions of the accessed
     * net has changed since the last call to <code>getTransitionIDs()
     * </code>.
     * </p>
     *
     * @see NetAccessor#getTransition
     */
    public TransitionAccessor getTransition(NetElementID id)
            throws RemoteException {
        Transition result = ((Net) object).getTransitionWithID(id);

        if (result == null) {
            return null;
        }
        return new TransitionAccessorImpl(result, getEnvironment());
    }
}