package de.renew.shadow;

import de.renew.net.Net;
import de.renew.net.Place;
import de.renew.net.Transition;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;


/**
 * This is simply a typesafe hash table to identify the
 * compiled objects that arise from shadow objects.
 **/
public class ShadowLookup {
    private Hashtable netMap;
    private Hashtable placeMap;
    private Hashtable transitionMap;
    private Hashtable extensions;

    public ShadowLookup() {
        netMap = new Hashtable();
        placeMap = new Hashtable();
        transitionMap = new Hashtable();
        extensions = new Hashtable();
    }

    public void setNet(String name, Net net) {
        netMap.put(name, net);
    }

    public Net getNet(String name) {
        return (Net) netMap.get(name);
    }

    public Enumeration allNetNames() {
        return netMap.keys();
    }

    public void set(ShadowPlace element, Place obj) {
        placeMap.put(element, obj);
    }

    public Place get(ShadowPlace element) {
        return (Place) placeMap.get(element);
    }

    public Enumeration allPlaces() {
        return placeMap.keys();
    }

    public void set(ShadowTransition element, Transition obj) {
        transitionMap.put(element, obj);
    }

    public Transition get(ShadowTransition element) {
        return (Transition) transitionMap.get(element);
    }

    public Enumeration allTransitions() {
        return transitionMap.keys();
    }

    /**
     * Return an extension to this shadow lookup
     * for the extension category specified by
     * the given factory. If no extension of said
     * category is currently available, use the
     * factory to create an instance.
     * On multiple calls with factories for the same
     * category, this method will consistently
     * return the same object.
     **/
    public synchronized ShadowLookupExtension getShadowLookupExtension(ShadowLookupExtensionFactory factory) {
        ShadowLookupExtension extension = (ShadowLookupExtension) extensions
                                          .get(factory.getCategory());
        if (extension == null) {
            extension = factory.createExtension();
            extensions.put(factory.getCategory(), extension);
        }
        return extension;
    }


    /**
     * Publicly announces all compiled nets. This has to be done
     * at the end of the compilation process.
     **/
    public void makeNetsKnown() {
        Iterator nets = netMap.values().iterator();
        while (nets.hasNext()) {
            ((Net) nets.next()).makeKnown();
        }
    }
}